/*
 *	SYS_UNIX -- runtime support routines for T3X under Unix.
 *	Copyright (C) 1999,2000 Nils M Holm. All rights reserved.
 *	See the file LICENSE for conditions of use.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <fcntl.h>

#define uchar	unsigned char
#define ushort	unsigned short
#define uint	unsigned int
#define ulong	unsigned long


#define BPW	2

#define NDIR	64
DIR	*Dirfd[NDIR];

#define MAXARGS	128

#define RDOK	0x01
#define WROK	0x02
#define EXOK	0x04
#define ISDIR	0x08


sys_init() {
	int	i;

	for (i=0; i<NDIR; i++) Dirfd[i] = NULL;
}


sys_pipe(fds)
int	*fds;
{
	int	a[2], r;

	r = pipe(a);
	fds[0] = a[0];
	fds[1] = a[1];
	return r;
}


sys_dup(fd) {
	return dup(fd);
}


sys_dup2(ofd, nfd) {
	return dup2(ofd, nfd);
}


sys_rdchk(fd) {
	struct fd_set	fdset;
	struct timeval	z;

	FD_ZERO(&fdset);
	FD_SET(fd, &fdset);
	z.tv_sec = z.tv_usec = 0L;
	return (select(fd+1, &fdset, NULL, NULL, &z) == 1)? -1: 0;
}


sys_mkdir(s)
char	*s;
{
	return mkdir(s, 0755);
}


sys_chdir(s)
char	*s;
{
	return chdir(s);
}


sys_rmdir(s)
char	*s;
{
	return rmdir(s);
}


sys_getdir(buf, len)
char	*buf;
int	len;
{
	if (len < 65) return -1;
	if (getcwd(buf, len) < 0) return -1;
	return strlen(buf);
}


sys_opendir(s)
char	*s;
{
	int	i;

	for (i=0; i<NDIR; i++) if (Dirfd[i] == NULL) break;
	if (i >= NDIR) return -1;
	if ((Dirfd[i] = opendir(s)) == NULL) return -1;
	return i;
}


sys_readdir(dd, b, n)
int	dd;
char	*b;
int	n;
{
	struct dirent	*dp;
	int		k;

	if (Dirfd[dd] == NULL) return -1;
	dp = readdir(Dirfd[dd]);
	if (dp == NULL) return -1;
	k = dp->d_namlen;
	k = k>=n? n-1: k;
	memcpy(b, dp->d_name, k);
	b[k] = 0;
	return k;
}


sys_closedir(dd) {
	DIR	*old;

	if (Dirfd[dd] == 0) return -1;
	old = Dirfd[dd];
	Dirfd[dd] = 0;
	return closedir(old);
}


accessmap(sp)
struct stat	*sp;
{
	int	uid = getuid(), gid = getgid();
	int	bits = 0;

	if (uid == 0) bits = RDOK|WROK;
	if (sp->st_uid == uid) {
		if (sp->st_mode & S_IRUSR) bits |= RDOK;
		if (sp->st_mode & S_IWUSR) bits |= WROK;
		if (sp->st_mode & S_IXUSR) bits |= EXOK;
	}
	if (sp->st_gid == gid) {
		if (sp->st_mode & S_IRGRP) bits |= RDOK;
		if (sp->st_mode & S_IWGRP) bits |= WROK;
		if (sp->st_mode & S_IXGRP) bits |= EXOK;
	}
	if (sp->st_mode & S_IROTH) bits |= RDOK;
	if (sp->st_mode & S_IWOTH) bits |= WROK;
	if (sp->st_mode & S_IXOTH) bits |= EXOK;
	return bits;
}


sys_stat(name, b)
char	*name;
short	*b;
{
	struct t3x_stat {
		short	dev;
		short	ino;
		short	mode;
		short	nlink;
		short	uid;
		short	gid;
		short	rdev;
		short	size;
		short	ext64;
		short	mt_year;
		short	mt_mon_day;
		short	mt_hr_min;
		short	mt_sec_hs;
	};
	struct stat	st;
	struct tm	*tp;
	uchar		*bb = (uchar *)b;

	if (stat(name, &st) < 0) return -1;
	b[0] = st.st_dev;
	b[1] = st.st_ino;
	b[2] = (st.st_mode & S_IFDIR? ISDIR: 0) | accessmap(&st);
	b[3] = st.st_nlink;
	b[4] = st.st_uid;
	b[5] = st.st_gid;
	b[6] = st.st_rdev;
	b[7] = (short) st.st_size;
	b[8] = (short) st.st_size / (64L*1024);
	tp = localtime(&st.st_mtime);
	bb[18] = (tp->tm_year+1900) / 100;
	bb[19] = tp->tm_year % 100;
	bb[20] = tp->tm_mon+1;
	bb[21] = tp->tm_mday;
	bb[22] = tp->tm_hour;
	bb[23] = tp->tm_min;
	bb[24] = tp->tm_sec;
	bb[25] = 0;
	return 0;
}


sys_wait () {
	int	r;

	wait(&r);
	return WEXITSTATUS(r);
}


sys_spawn(p, av, w)
char	*p;
char	**av;
int	w;
{
	int	r, i;
	char	*av2[MAXARGS];

	for (i = 0; av[i]; i++)
		if (i >= MAXARGS-1) return -1;
	for (++i; i>0; i--) av2[i] = av[i-1];
	av2[0] = p;
	if ((r = fork()) < 0) return -1;
	if (r) {
		if (w) return sys_wait();
	}
	else {
		execv(p, av2);
		exit(-1);
	}
	return r;
}


sys_fork () {
	return fork();
}


sys_kill(pid, n) {
	return kill(pid, n);
}


convtime(t, b)
time_t	t;
char	*b;
{
	int	mlen[] = { 0, 31,28,31,30,31,30,31,31,30,31,30,31 };
	int	yr, mon, day, hr, min, sec;
	long	dt;
	int	i;

	yr = 1970;
	while (1) {
		dt = (!(yr&3)? 366: 365) * 86400L;
		if (t - dt < 0) break;
		t -= dt;
		yr++;
	}
	mon = 1;
	mlen[2] = !(yr&3)? 29: 28;
	while (mon<12) {
		dt = mlen[mon] * 86400L;
		if (t - dt < 0) break;
		t -= dt;
		mon++;
	}
	day = (int) (t / 86400L + 1);
	t = (int) (t % 86400L);
	hr = (int) (t / 3600);
	t = (int) (t % 3600);
	min = (int) (t / 60);
	sec = (int) (t % 60);
	b[0] = yr / 100;
	b[1] = yr % 100;
	b[2] = mon;
	b[3] = day;
	b[4] = hr;
	b[5] = min;
	b[6] = sec;
	b[7] = 0;
}


/***********
maketime(b, tp)
char	*b;
int	*tp;
{
	int	mlen[] = { 0, 31,28,31,30,31,30,31,31,30,31,30,31 };
	int	yr, mon, day, hr, min, sec;
	int	i;
	long	t;

	yr = b[0] * 100 + b[1];
	mon = b[2];
	day = b[3];
	hr = b[4];
	min = b[5];
	sec = b[6];
	t = ((yr-1968)*365 + (yr-1968)/4 - 365*2) * 86400L;
	for (i=1; i<mon; i++) t += mlen[i] * 86400L;
	t += (day-1) * 86400L;
	t += hr * 3600;
	t += min * 60;
	t += sec;
	*tp = t;
	return 0;
}
***********/


sys_time(b)
char	*b;
{
	time_t	t;

	time(&t);
	convtime(t, b);
	return 0;
}

